@@ -99,7 +99,7 @@ proc Httpd_Init {} {
     }
     # We always have the counter module and mimetyes
     Counter_Init
-    Mtype_ReadTypes $Httpd(library)/mime.types
+    Mtype_ReadTypes [file join $Httpd(library) mime.types]
 }
 
 # Httpd_Server
@@ -135,7 +135,6 @@ proc Httpd_Server {{port 80} {name {}} {ipaddr {}}} {
     if {[catch $cmd Httpd(listen)]} {
 	return -code error "$Httpd(name):$port $Httpd(listen)"
     }
-    Counter_Reset accepts
 }
 
 # Kill the server gracefully
@@ -242,6 +241,18 @@ proc HttpdRead {sock} {
 	    1,start	{
 		if [regexp {^([^ ]+) +([^?]+)\??([^ ]*) +HTTP/(1.[01])} \
 			$line x data(proto) data(url) data(query) data(version)] {
+		    # Strip leading http://server.
+		    # We check and discard proxy requests here, too.
+
+		    if {[regexp {^http://([^/:]+)} $data(url) x xserv]} {
+			if {[string compare \
+				[string tolower $xserv] \
+				[string tolower $Httpd(name)]] != 0} {
+			    Httpd_Error $sock 400 $line
+			    return
+			}
+			regsub {^http://([^/]+)} $data(url) {} data(url)
+		    }
 		    set data(state) mime
 		    set data(line) $line
 		    set data(uri) $data(url)
@@ -331,7 +342,6 @@ proc HttpdRead {sock} {
 	Url_PostHook $sock $data(count)
 	Url_Dispatch $sock
     } else {
-	Httpd_Log $sock Error "Broken connection reading POST data"
 	Httpd_SockClose $sock 1 "broken connection during post data"
     }
 }
@@ -376,6 +386,9 @@ proc Httpd_GetPostData {sock varName {size -1}} {
     set block [read $sock $size]
     append buffer $block
     set data(count) [expr {$data(count) - [string length $block]}]
+    if {[eof $sock]} {
+	set data(count) 0
+    }
     return $data(count)
 }
 
@@ -541,6 +554,8 @@ proc HttpdSetCookie {sock} {
 	foreach item $data(set-cookie) {
 	    puts $sock "Set-Cookie: $item"
 	}
+	# HttpdCookieLog $sock HttpdSetCookie
+	unset data(set-cookie)
     }
 }
 
@@ -851,3 +866,30 @@ proc Httpd_SockClose {sock closeit {message Close}} {
 	    catch {puts stderr $msg}
 	}
     }
+
+proc HttpdCookieLog {sock what} {
+    global Log Httpd
+    upvar #0 Httpd$sock data
+    if {[info exist Log(log)] && ![info exist Httpd(cookie_log)]} {
+	set Httpd(cookie_log) [open $Log(log)cookie a]
+    }
+    if {[info exist Httpd(cookie_log)]} {
+	append result [LogValue data(ipaddr)]
+	append result { } \[[clock format [clock seconds] -format %d/%h/%Y:%T] -0700\]
+
+	append result { } $what
+	switch $what {
+	    Url_Dispatch {
+		if {![info exist data(mime,cookie)]} {
+		    return
+		}
+		append result { } \"$data(mime,cookie)\"
+	    }
+	    Httpd_SetCookie -
+	    HttpdSetCookie {
+		append result { } \"[LogValue data(set-cookie)]\"
+	    }
+	}
+	catch { puts $Httpd(cookie_log) $result ; flush $Httpd(cookie_log)}
+    }
+}
